#include <errno.h>
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <limits.h>

#include <iostream>

int remove_links(const std::string path, const std::string dirl, int timer)
{
  // removes hard links in directory "dirl" to file "path" 
  // in directory "dir", if they are older than "time" seconds. 
  // Method: 
  // using readdir, examine all components of dirl and see
  // if they have the same inode as "path". If so, remove, unless
  // the realname is the same as the realname of "path"
  //
  DIR *pdir;
  struct dirent *pent;
  int rc;
  struct stat buf;

  pdir = opendir(dirl.c_str());
  if (!pdir)
  {
    std::cerr << "Cannot read link directory '"<<dirl<<"'"<<std::endl;
    return 1;
  }
  rc = stat(path.c_str(),&buf);
  if (rc !=0)
  {
    std::cerr << "Cannot stat '"<<path<<"'"<<std::endl;
    return 1;
  }
  ino_t inodenr = buf.st_ino;
  char rpathc[PATH_MAX+1];
  realpath(path.c_str(),rpathc);
  std::string rpath = rpathc;
  std::cerr<<"TD: basefile '"<<rpath<<"'"<<std::endl;
  std::string rpath1;
  
  while ((pent=readdir(pdir)))
  {
    struct stat buf;
    std::string name = pent->d_name;
    if (name == ".") continue;
    if (name == "..") continue;
    realpath((dirl+"/"+name).c_str(),rpathc);
    rpath1 = rpathc;
    std::cerr<<"TD: looking at '"<<rpath1<<"'"<<std::endl;
    if (rpath != rpath1)
    {
      int rc = stat(rpath1.c_str(),&buf);
      if (rc != 0)
      {
	std::cerr<<"Cannot stat '"<<rpath1<<"'"<<" this cannot happen!" << std::endl;
	return 1;
      }
      if (buf.st_ino == inodenr)
      {
	std::cerr << "time: " << time(0) << " " << buf.st_mtime << " " << timer << std::endl;
	if (time(0) > buf.st_mtime + timer)
	{
	  int rc = unlink(rpath1.c_str());
	  if (rc == 0)
	    std::cerr<<"Unlinked '"<<rpath1<<"'"<<std::endl;
	  else
	    std::cerr<<"Could not unlink '"<<rpath1<<"'"<<std::endl;
	}
	else
	  std::cerr << "TD: not unlinking because time: '"<<rpath1<<"'"<<std::endl;
      }
    }
    else
      std::cerr<<"TD: not unlinking '"<<rpath1<<"'"<<std::endl;
  }
  return 0;
}
int main()
{
  std::string poolfilename="pool";
  std::string linkdirname="linkdir";
  int timer = 60;
  remove_links(poolfilename, linkdirname,timer);
  return 0;
}
